@@ -1,7 +1,7 @@
 #!/usr/bin/python2
 # -*- coding: utf-8 -*-
 
-# :Id: $Id: latex2mathml.py 8554 2020-09-04 16:52:11Z milde $
+# :Id: $Id: latex2mathml.py 8675 2021-04-08 16:35:54Z milde $
 # :Copyright: © 2010 Günter Milde.
 #             Based on rst2mathml.py from the latex_math sandbox project
 #             © 2005 Jens Jørgen Mortensen
@@ -524,7 +524,7 @@ def handle_keyword(name, node, string):
         node = node.append(mo(negatables[operator]))
         skip += len(operator)
     elif name == 'mathbf':
-        style = mstyle(nchildren=1, fontweight='bold')
+        style = mstyle(nchildren=1, mathvariant='bold')
         node.append(style)
         node = style
     elif name == 'mathbb':
