@@ -1,7 +1,7 @@
 #!/usr/bin/python2
 # -*- coding: utf-8 -*-
 
-# :Id: $Id: smartquotes.py 8554 2020-09-04 16:52:11Z milde $
+# :Id: $Id: smartquotes.py 8679 2021-04-09 11:58:10Z grubert $
 # :Copyright: © 2010 Günter Milde,
 #             original `SmartyPants`_: © 2003 John Gruber
 #             smartypants.py:          © 2004, 2007 Chad Miller
@@ -636,7 +636,7 @@ def educateQuotes(text, language='en'):
     """
 
     smart = smartchars(language)
-    ch_classes = {'open': u'[(\[{]', # opening braces
+    ch_classes = {'open': u'[([{]', # opening braces
                   'close': r'[^\s]', # everything except whitespace
                   'punct': r"""[-!"#\$\%'()*+,.\/:;<=>?\@\[\\\]\^_`{|}~]""",
                   'dash': u'[-–—]' # hyphen and em/en dashes
