@@ -3,7 +3,7 @@
 # :Author: David Goodger, Günter Milde
 #          Based on the html4css1 writer by David Goodger.
 # :Maintainer: docutils-develop@lists.sourceforge.net
-# :Revision: $Revision: 8118 $
+# :Revision: $Revision: 8644 $
 # :Date: $Date: 2005-06-28$
 # :Copyright: © 2016 David Goodger, Günter Milde
 # :License: Released under the terms of the `2-Clause BSD license`_, in short:
@@ -13,14 +13,15 @@
 #    notice and this notice are preserved.
 #    This file is offered as-is, without any warranty.
 #
-# .. _2-Clause BSD license: http://www.spdx.org/licenses/BSD-2-Clause
+# .. _2-Clause BSD license: https://opensource.org/licenses/BSD-2-Clause
 
 """common definitions for Docutils HTML writers"""
 
-import sys
-import os.path
+import base64
+import mimetypes
+import os, os.path
 import re
-import urllib
+import sys
 
 try: # check for the Python Imaging Library
     import PIL.Image
@@ -39,6 +40,14 @@ from docutils.transforms import writer_aux
 from docutils.utils.math import (unichar2tex, pick_math_environment,
                                  math2html, latex2mathml, tex2mathml_extern)
 
+if sys.version_info >= (3, 0):
+    from urllib.request import url2pathname
+else:
+    from urllib import url2pathname
+
+if sys.version_info >= (3, 0):
+    unicode = str  # noqa
+
 
 class Writer(writers.Writer):
 
@@ -54,7 +63,7 @@ class Writer(writers.Writer):
     settings_defaults = {'output_encoding_error_handler': 'xmlcharrefreplace'}
 
     # config_section = ... # set in subclass!
-    config_section_dependencies = ['writers', 'html writers']
+    config_section_dependencies = ('writers', 'html writers')
 
     visitor_attributes = (
         'head_prefix', 'head', 'stylesheet', 'body_prefix',
@@ -159,7 +168,7 @@ class HTMLTranslator(nodes.NodeVisitor):
 
     head_prefix_template = ('<html xmlns="http://www.w3.org/1999/xhtml"'
                             ' xml:lang="%(lang)s" lang="%(lang)s">\n<head>\n')
-    content_type = ('<meta charset="%s"/>\n')
+    content_type = '<meta charset="%s"/>\n'
     generator = ('<meta name="generator" content="Docutils %s: '
                  'http://docutils.sourceforge.net/" />\n')
 
@@ -183,7 +192,7 @@ class HTMLTranslator(nodes.NodeVisitor):
 
     stylesheet_link = '<link rel="stylesheet" href="%s" type="text/css" />\n'
     embedded_stylesheet = '<style type="text/css">\n\n%s\n</style>\n'
-    words_and_spaces = re.compile(r'\S+| +|\n')
+    words_and_spaces = re.compile(r'[^ \n]+| +|\n')
     # wrap point inside word:
     in_word_wrap_point = re.compile(r'.+\W\W.+|[-?].+', re.U)
     lang_attribute = 'lang' # name changes to 'xml:lang' in XHTML 1.1
@@ -235,7 +244,7 @@ class HTMLTranslator(nodes.NodeVisitor):
         Used by visit_* and depart_* functions in conjunction with the tree
         traversal. Make sure that the pops correspond to the pushes."""
 
-        self.topic_classes = [] # TODO: replace with self_in_contents
+        self.topic_classes = []
         self.colspecs = []
         self.compact_p = True
         self.compact_simple = False
@@ -302,8 +311,8 @@ class HTMLTranslator(nodes.NodeVisitor):
                 content = io.FileInput(source_path=path,
                                        encoding='utf-8').read()
                 self.settings.record_dependencies.add(path)
-            except IOError, err:
-                msg = u"Cannot embed stylesheet '%s': %s." % (
+            except IOError as err:
+                msg = u"Cannot embed stylesheet '%r': %s." % (
                                 path, SafeString(err.strerror))
                 self.document.reporter.error(msg)
                 return '<--- %s --->\n' % msg
@@ -328,7 +337,7 @@ class HTMLTranslator(nodes.NodeVisitor):
         classes = []
         languages = []
         # unify class arguments and move language specification
-        for cls in node.get('classes', []) + atts.pop('class', '').split() :
+        for cls in node.get('classes', []) + atts.pop('class', '').split():
             if cls.startswith('language-'):
                 languages.append(cls[9:])
             elif cls.strip() and cls not in classes:
@@ -362,8 +371,7 @@ class HTMLTranslator(nodes.NodeVisitor):
                     # Non-empty tag.  Place the auxiliary <span> tag
                     # *inside* the element, as the first child.
                     suffix += '<span id="%s"></span>' % id
-        attlist = atts.items()
-        attlist.sort()
+        attlist = sorted(atts.items())
         parts = [tagname]
         for name, value in attlist:
             # value=None was used for boolean attributes without
@@ -494,7 +502,6 @@ class HTMLTranslator(nodes.NodeVisitor):
     # the end of this file).
 
     def is_compactable(self, node):
-        # print "is_compactable %s ?" % node.__class__,
         # explicite class arguments have precedence
         if 'compact' in node['classes']:
             return True
@@ -503,14 +510,13 @@ class HTMLTranslator(nodes.NodeVisitor):
         # check config setting:
         if (isinstance(node, (nodes.field_list, nodes.definition_list))
             and not self.settings.compact_field_lists):
-            # print "`compact-field-lists` is False"
             return False
         if (isinstance(node, (nodes.enumerated_list, nodes.bullet_list))
             and not self.settings.compact_lists):
-            # print "`compact-lists` is False"
             return False
-        # more special cases:
-        if (self.topic_classes == ['contents']): # TODO: self.in_contents
+        # Table of Contents:
+        if (self.topic_classes == ['contents']):
+            # TODO: look in parent nodes, remove self.topic_classes?
             return True
         # check the list items:
         return self.check_simple_list(node)
@@ -535,14 +541,15 @@ class HTMLTranslator(nodes.NodeVisitor):
     def depart_caption(self, node):
         self.body.append('</p>\n')
 
-    # citations
-    # ---------
-    # Use definition list instead of table for bibliographic references.
-    # Join adjacent citation entries.
-
     def visit_citation(self, node):
+        # Use definition list for bibliographic references.
+        # Join adjacent citation entries.
+        # TODO: use <aside>.
         if not self.in_footnote_list:
-            self.body.append('<dl class="citation">\n')
+            listnode = node.copy()
+            listnode['ids'] = []
+            self.body.append(self.starttag(listnode, 'dl', CLASS='citation'))
+            # self.body.append('<dl class="citation">\n')
             self.in_footnote_list = True
 
     def depart_citation(self, node):
@@ -681,6 +688,7 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.body.append('</dd>\n')
 
     def visit_docinfo(self, node):
+        self.context.append(len(self.body))
         classes = 'docinfo'
         if (self.is_compactable(node)):
             classes += ' simple'
@@ -688,6 +696,9 @@ class HTMLTranslator(nodes.NodeVisitor):
 
     def depart_docinfo(self, node):
         self.body.append('</dl>\n')
+        start = self.context.pop()
+        self.docinfo = self.body[start:]
+        self.body = []
 
     def visit_docinfo_item(self, node, name, meta=True):
         if meta:
@@ -720,6 +731,9 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.html_prolog.append(self.doctype)
         self.meta.insert(0, self.content_type % self.settings.output_encoding)
         self.head.insert(0, self.content_type % self.settings.output_encoding)
+        if 'name="dcterms.' in ''.join(self.meta):
+            self.head.append(
+             '<link rel="schema.dcterms" href="http://purl.org/dc/terms/"/>')
         if self.math_header:
             if self.math_output == 'mathjax':
                 self.head.extend(self.math_header)
@@ -843,15 +857,16 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.body_suffix[:0] = footer
         del self.body[start:]
 
-    # footnotes
-    # ---------
-    # use definition list instead of table for footnote text
-
-    # TODO: use the new HTML5 element <aside>? (Also for footnote text)
+    # TODO: use the new HTML5 element <aside> for footnote text
+    # (allows better styling with CSS, the current <dl> list styling
+    # with "float" interferes with sidebars).
     def visit_footnote(self, node):
         if not self.in_footnote_list:
+            listnode = node.copy()
+            listnode['ids'] = []
             classes = 'footnote ' + self.settings.footnote_references
-            self.body.append('<dl class="%s">\n'%classes)
+            self.body.append(self.starttag(listnode, 'dl', CLASS=classes))
+            # self.body.append('<dl class="%s">\n'%classes)
             self.in_footnote_list = True
 
     def depart_footnote(self, node):
@@ -875,7 +890,6 @@ class HTMLTranslator(nodes.NodeVisitor):
         if 'sectnum' in node['classes']:
             # get section number (strip trailing no-break-spaces)
             sectnum = node.astext().rstrip(u' ')
-            # print sectnum.encode('utf-8')
             self.body.append('<span class="sectnum">%s</span> '
                                     % self.encode(sectnum))
             # Content already processed:
@@ -896,19 +910,10 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.header.extend(header)
         del self.body[start:]
 
-    # Image types to place in an <object> element
-    object_image_types = {'.swf': 'application/x-shockwave-flash'}
-
     def visit_image(self, node):
         atts = {}
         uri = node['uri']
-        ext = os.path.splitext(uri)[1].lower()
-        if ext in self.object_image_types:
-            atts['data'] = uri
-            atts['type'] = self.object_image_types[ext]
-        else:
-            atts['src'] = uri
-            atts['alt'] = node.get('alt', uri)
+        mimetype = mimetypes.guess_type(uri)[0]
         # image size
         if 'width' in node:
             atts['width'] = node['width']
@@ -917,7 +922,7 @@ class HTMLTranslator(nodes.NodeVisitor):
         if 'scale' in node:
             if (PIL and not ('width' in node and 'height' in node)
                 and self.settings.file_insertion_enabled):
-                imagepath = urllib.url2pathname(uri)
+                imagepath = url2pathname(uri)
                 try:
                     img = PIL.Image.open(
                             imagepath.encode(sys.getfilesystemencoding()))
@@ -957,15 +962,41 @@ class HTMLTranslator(nodes.NodeVisitor):
             suffix = '\n'
         if 'align' in node:
             atts['class'] = 'align-%s' % node['align']
-        if ext in self.object_image_types:
+        # Embed image file (embedded SVG or data URI):
+        if self.settings.embed_images or ('embed' in node):
+            err_msg = ''
+            if not mimetype:
+                err_msg = 'unknown MIME type'
+            if not self.settings.file_insertion_enabled:
+                err_msg = 'file insertion disabled.'
+            try:
+                with open(url2pathname(uri), 'rb') as imagefile:
+                    imagedata = imagefile.read()
+            except IOError as err:
+                err_msg = err.strerror
+            if err_msg:
+                self.document.reporter.error('Cannot embed image %r: %s'
+                                             %(uri, err_msg))
+            else:
+                self.settings.record_dependencies.add(
+                                            uri.replace('\\', '/'))
+                # TODO: insert SVG as-is?
+                # if mimetype == 'image/svg+xml':
+                  # read/parse, apply arguments,
+                  # insert as <svg ....> ... </svg> # (about 1/3 less data)
+                data64 = base64.b64encode(imagedata).decode()
+                uri = u'data:%s;base64,%s' % (mimetype, data64)
+        if mimetype == 'application/x-shockwave-flash':
+            atts['type'] = mimetype
             # do NOT use an empty tag: incorrect rendering in browsers
-            self.body.append(self.starttag(node, 'object', suffix, **atts) +
-                             node.get('alt', uri) + '</object>' + suffix)
+            tag = (self.starttag(node, 'object', '', data=uri, **atts)
+                   + node.get('alt', uri) + '</object>' + suffix)
         else:
-            self.body.append(self.emptytag(node, 'img', suffix, **atts))
+            atts['alt'] = node.get('alt', node['uri'])
+            tag = self.emptytag(node, 'img', suffix, src=uri, **atts)
+        self.body.append(tag)
 
     def depart_image(self, node):
-        # self.body.append(self.context.pop())
         pass
 
     def visit_inline(self, node):
@@ -997,9 +1028,8 @@ class HTMLTranslator(nodes.NodeVisitor):
                 self.body.append('</a>')
         self.body.append('</span>')
         if self.settings.footnote_backlinks and len(backrefs) > 1:
-            # Python 2.4 fails with enumerate(backrefs, 1)
-            backlinks = ['<a href="#%s">%s</a>' % (ref, i+1)
-                            for (i, ref) in enumerate(backrefs)]
+            backlinks = ['<a href="#%s">%s</a>' % (ref, i)
+                            for (i, ref) in enumerate(backrefs, 1)]
             self.body.append('<span class="fn-backref">(%s)</span>'
                                 % ','.join(backlinks))
         self.body.append('</dt>\n<dd>')
@@ -1157,7 +1187,7 @@ class HTMLTranslator(nodes.NodeVisitor):
                     'with math-output "MathML"')
             except OSError:
                     raise OSError('is "latexmlmath" in your PATH?')
-            except SyntaxError, err:
+            except SyntaxError as err:
                 err_node = self.document.reporter.error(err, base_node=node)
                 self.visit_system_message(err_node)
                 self.body.append(self.starttag(node, 'p'))
@@ -1188,7 +1218,6 @@ class HTMLTranslator(nodes.NodeVisitor):
         pass # never reached
 
     def visit_math_block(self, node):
-        # print node.astext().encode('utf8')
         math_env = pick_math_environment(node.astext())
         self.visit_math(node, math_env=math_env)
 
@@ -1317,6 +1346,8 @@ class HTMLTranslator(nodes.NodeVisitor):
                    'References must have "refuri" or "refid" attribute.'
             atts['href'] = '#' + node['refid']
             atts['class'] += ' internal'
+        if len(node) == 1 and isinstance(node[0], nodes.image):
+            atts['class'] += ' image-reference'
         if not isinstance(node.parent, nodes.TextElement):
             assert len(node) == 1 and isinstance(node[0], nodes.image)
             atts['class'] += ' image-reference'
@@ -1347,7 +1378,6 @@ class HTMLTranslator(nodes.NodeVisitor):
     def depart_rubric(self, node):
         self.body.append('</p>\n')
 
-    # TODO: use the new HTML 5 element <section>?
     def visit_section(self, node):
         self.section_level += 1
         self.body.append(
@@ -1357,7 +1387,7 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.section_level -= 1
         self.body.append('</div>\n')
 
-    # TODO: use the new HTML5 element <aside>? (Also for footnote text)
+    # TODO: use the new HTML5 element <aside>
     def visit_sidebar(self, node):
         self.body.append(
             self.starttag(node, 'div', CLASS='sidebar'))
@@ -1401,14 +1431,14 @@ class HTMLTranslator(nodes.NodeVisitor):
             classes = 'sidebar-subtitle'
         elif isinstance(node.parent, nodes.document):
             classes = 'subtitle'
-            self.in_document_title = len(self.body)
+            self.in_document_title = len(self.body)+1
         elif isinstance(node.parent, nodes.section):
             classes = 'section-subtitle'
         self.body.append(self.starttag(node, 'p', '', CLASS=classes))
 
     def depart_subtitle(self, node):
         self.body.append('</p>\n')
-        if self.in_document_title:
+        if isinstance(node.parent, nodes.document):
             self.subtitle = self.body[self.in_document_title:-1]
             self.in_document_title = 0
             self.body_pre_docinfo.extend(self.body)
@@ -1450,16 +1480,15 @@ class HTMLTranslator(nodes.NodeVisitor):
     def depart_system_message(self, node):
         self.body.append('</div>\n')
 
-    # tables
-    # ------
-    # no hard-coded border setting in the table head::
-
     def visit_table(self, node):
+        atts = {}
         classes = [cls.strip(u' \t\n')
                    for cls in self.settings.table_style.split(',')]
         if 'align' in node:
             classes.append('align-%s' % node['align'])
-        tag = self.starttag(node, 'table', CLASS=' '.join(classes))
+        if 'width' in node:
+            atts['style'] = 'width: %s' % node['width']
+        tag = self.starttag(node, 'table', CLASS=' '.join(classes), **atts)
         self.body.append(tag)
 
     def depart_table(self, node):
@@ -1508,11 +1537,10 @@ class HTMLTranslator(nodes.NodeVisitor):
 
     def visit_title(self, node):
         """Only 6 section levels are supported by HTML."""
-        check_id = 0  # TODO: is this a bool (False) or a counter?
         close_tag = '</p>\n'
         if isinstance(node.parent, nodes.topic):
             self.body.append(
-                  self.starttag(node, 'p', '', CLASS='topic-title first'))
+                  self.starttag(node, 'p', '', CLASS='topic-title'))
         elif isinstance(node.parent, nodes.sidebar):
             self.body.append(
                   self.starttag(node, 'p', '', CLASS='sidebar-title'))
@@ -1562,17 +1590,13 @@ class HTMLTranslator(nodes.NodeVisitor):
     def depart_title_reference(self, node):
         self.body.append('</cite>')
 
-    # TODO: use the new HTML5 element <aside>? (Also for footnote text)
     def visit_topic(self, node):
         self.body.append(self.starttag(node, 'div', CLASS='topic'))
         self.topic_classes = node['classes']
-        # TODO: replace with ::
-        #   self.in_contents = 'contents' in node['classes']
 
     def depart_topic(self, node):
         self.body.append('</div>\n')
         self.topic_classes = []
-        # TODO self.in_contents = False
 
     def visit_transition(self, node):
         self.body.append(self.emptytag(node, 'hr', CLASS='docutils'))
@@ -1606,20 +1630,16 @@ class SimpleListChecker(nodes.GenericNodeVisitor):
         raise nodes.NodeFound
 
     def visit_list_item(self, node):
-        # print "visiting list item", node.__class__
         children = [child for child in node.children
                     if not isinstance(child, nodes.Invisible)]
-        # print "has %s visible children" % len(children)
         if (children and isinstance(children[0], nodes.paragraph)
             and (isinstance(children[-1], nodes.bullet_list) or
                  isinstance(children[-1], nodes.enumerated_list) or
                  isinstance(children[-1], nodes.field_list))):
             children.pop()
-        # print "%s children remain" % len(children)
         if len(children) <= 1:
             return
         else:
-            # print "found", child.__class__, "in", node.__class__
             raise nodes.NodeFound
 
     def pass_node(self, node):
